/*
 base64 - an XOP designed to encode/decode base64 strings
 */
//string check = base64encode("weldon")

#include "base64.h"
#include <curl/curl.h>
#include "md5wrapper.h"
#include <string>
#include <iostream>

#include "error.h"
#include <stdio.h>
#include <stdlib.h>

using namespace std;

#ifdef _MACINTOSH_
HOST_IMPORT int main(IORecHandle ioRecHandle);
#endif	
#ifdef _WINDOWS_
HOST_IMPORT void main(IORecHandle ioRecHandle);
#endif

static long
RegisterFunction()
{
	int funcIndex;
	
	funcIndex = GetXOPItem(0);			// Which function invoked ?
	switch (funcIndex) {
		case 0:							
			return((long)base64encode);	// This function is called using the direct method.
			break;
		case 1:
			return((long)base64decode);
			break;
		case 2:
			return((long)URLencode);
			break;
		case 3:
			return((long)md5hash);
			break;
	}
	return NIL;
}

/*	XOPEntry()
 
 This is the entry point from the host application to the XOP for all
 messages after the INIT message.
 */
static void
XOPEntry(void)
{	
	long result = 0;
	
	switch (GetXOPMessage()) {
		case FUNCADDRS:
			result = RegisterFunction();	// This tells Igor the address of our function.
			break;
	}
	SetXOPResult(result);
}

/*	main(ioRecHandle)
 
 This is the initial entry point at which the host application calls XOP.
 The message sent by the host must be INIT.
 main() does any necessary initialization and then sets the XOPEntry field of the
 ioRecHandle to the address to be called for future messages.
 */
#ifdef _MACINTOSH_
HOST_IMPORT int main(IORecHandle ioRecHandle)
#endif	
#ifdef _WINDOWS_
HOST_IMPORT void main(IORecHandle ioRecHandle)
#endif
{	
	XOPInit(ioRecHandle);							// Do standard XOP initialization.
	SetXOPEntry(XOPEntry);							// Set entry point for future calls.
	
	if (igorVersion < 504)
		SetXOPResult(REQUIRES_IGOR_504);
	else
		SetXOPResult(0L);
}

int base64encode(base64encoderStruct *p){
	int err = 0;
	
	Handle dest = NULL;	
	string encoded;
	
	if(p->src == NULL){
		err = NULL_STRING_HANDLE;
		p->dest = NULL;
		goto done;
	}
	
	encoded = base64_encode((const unsigned char*) *(p->src), GetHandleSize(p->src));
	
	dest = NewHandle(encoded.length());
	if(dest == NULL){
		err = NOMEM;
		goto done;
	}
	
	memcpy(*dest, encoded.data(), encoded.length());
	
	
done:
	
	if(p->src)
		DisposeHandle(p->src);
	
	p->dest = NULL;	// Init to NULL	
	if (err) {
		if (dest != NULL)
			DisposeHandle(dest);
		return err;
	}
	p->dest = dest;
	
	return err;
}

int base64decode(base64encoderStruct *p){
	int err = 0;
	
	Handle dest = NULL;
	string decoded;
	
	if(p->src == NULL){
		err = NULL_STRING_HANDLE;
		p->dest = NULL;
		goto done;
	}
	
	decoded = base64_decode(string(*(p->src), GetHandleSize(p->src)));
	
	dest = NewHandle(decoded.length());
	if(dest == NULL){
		err = NOMEM;
		goto done;
	}	

	memcpy(*dest, decoded.data(), decoded.length());
		
done:
	
	if(p->src)
		DisposeHandle(p->src);
	
	p->dest = NULL;	// Init to NULL	
	if (err) {
		if (dest != NULL)
			DisposeHandle(dest);
		return err;
	}
	p->dest = dest;
	
	return err;
}

int URLencode(base64encoderStruct *p){
	int err = 0;
	CURL *curl = NULL;
	char* output = NULL;
	char *src = NULL;
	Handle dest = NULL;

	dest = NewHandle(0);
	if(dest == NULL){
		err = NOMEM;
		goto done;
	}
	
	curl_global_init(CURL_GLOBAL_ALL);
	curl= curl_easy_init();
	if( !curl)
		goto done;
		
	if(p->src == NULL){
		err = NULL_STRING_HANDLE;
		p->dest = NULL;
		goto done;
	}
	
	src = (char*)malloc(1+sizeof(char)*GetHandleSize(p->src));
	if(src == NULL){
		err = NOMEM;
		goto done;
	}
	if(err = GetCStringFromHandle(p->src, src, GetHandleSize(p->src)))
		goto done;
		
	output = curl_easy_escape( curl , src  , 0 );	
		
	if(err = PutCStringInHandle(output,dest))
		goto done;
		
done:	
	if(src)
		free(src);
	if(p->src)
		DisposeHandle(p->src);
	
	if(output)
		curl_free(output);

	if(curl){
		//always cleanup
		curl_easy_cleanup(curl);
	}
	
	/* cleanup libcurl*/
	curl_global_cleanup();

	p->dest = NULL;	// Init to NULL	
	if (err) {
		if (dest != NULL)
			DisposeHandle(dest);
		return err;
	}
	p->dest = dest;
	return err;
	
}

int md5hash(base64encoderStruct *p){
	int err = 0;
	char *src = NULL;
	Handle dest = NULL;
	
	// creating a md5wrapper object
	md5wrapper md5;
	std:string hash1;

	dest = NewHandle(0);
	if(dest == NULL){
		err = NOMEM;
		goto done;
	}
	
	if(p->src == NULL){
		err = NULL_STRING_HANDLE;
		p->dest = NULL;
		goto done;
	}
	
	src = (char*)malloc(1+sizeof(char)*GetHandleSize(p->src));
	if(src == NULL){
		err = NOMEM;
		goto done;
	}
	
	if(err = GetCStringFromHandle(p->src, src, GetHandleSize(p->src)))
		goto done;

	// create a hash from a string
	hash1 = md5.getHashFromString(src);
	
	if(err = PutCStringInHandle(hash1.c_str(), dest))
		goto done;
		
done:
	if(src)
		free(src);
	if(p->src)
		DisposeHandle(p->src);
	
	p->dest = NULL;	// Init to NULL
	if (err) {
		if (dest != NULL)
			DisposeHandle(dest);
		return err;
	}
	p->dest = dest;
	
	return err;
}


